package dev.inmo.postssystem.features.files.server

import dev.inmo.postssystem.features.files.common.*
import dev.inmo.postssystem.features.files.common.storage.*
import dev.inmo.micro_utils.ktor.server.*
import dev.inmo.micro_utils.ktor.server.configurators.ApplicationRoutingConfigurator
import dev.inmo.micro_utils.repos.ktor.server.crud.configureReadStandardCrudRepoRoutes
import dev.inmo.micro_utils.repos.ktor.server.crud.configureWriteStandardCrudRepoRoutes
import io.ktor.application.call
import io.ktor.auth.authenticate
import io.ktor.response.respondBytes
import io.ktor.routing.*
import kotlinx.serialization.builtins.nullable

class FilesRoutingConfigurator(
    private val filesStorage: FilesStorage,
    private val writeFilesStorage: WriteFilesStorage?
) : ApplicationRoutingConfigurator.Element {
    constructor(fullFilesStorage: FullFilesStorage) : this(fullFilesStorage, fullFilesStorage)

    override fun Route.invoke() {
        authenticate {
            route(filesRootPathPart) {
                configureReadStandardCrudRepoRoutes(
                    filesStorage,
                    MetaFileInfoStorageWrapper.serializer(),
                    MetaFileInfoStorageWrapper.serializer().nullable,
                    FileId.serializer()
                )
                writeFilesStorage ?.let {
                    configureWriteStandardCrudRepoRoutes(
                        writeFilesStorage,
                        FullFileInfoStorageWrapper.serializer(),
                        FullFileInfoStorageWrapper.serializer().nullable,
                        FullFileInfo.serializer(),
                        FileId.serializer()
                    )
                }
                post(filesGetFilesPathPart) {
                    call.respondBytes(
                        filesStorage.getBytes(
                            call.uniload(FileId.serializer())
                        )
                    )
                }
                get(filesGetFullFileInfoPathPart) {
                    call.unianswer(
                        FullFileInfoStorageWrapper.serializer().nullable,
                        filesStorage.getFullFileInfo(
                            call.decodeUrlQueryValueOrSendError(filesFileIdParameter, FileId.serializer()) ?: return@get
                        )
                    )
                }
            }
        }
    }
}
