package dev.inmo.postssystem.features.roles.client

import dev.inmo.postssystem.features.roles.common.*
import dev.inmo.postssystem.features.users.common.UserId
import dev.inmo.micro_utils.ktor.client.UnifiedRequester
import dev.inmo.micro_utils.ktor.common.buildStandardUrl
import kotlinx.serialization.KSerializer
import kotlinx.serialization.builtins.ListSerializer
import kotlinx.serialization.builtins.serializer

class ReadClientUsersRolesStorage<T : UserRole>(
    private val baseUrl: String,
    private val unifiedRequester: UnifiedRequester,
    private val serializer: KSerializer<T>
) : ReadUsersRolesStorage<T> {
    private val userRolesSerializer = ListSerializer(serializer)

    private val userRolesFullUrl = buildStandardUrl(
        baseUrl,
        usersRolesRootPathPart
    )

    override suspend fun getUsers(
        userRole: T
    ): List<UserId> = unifiedRequester.uniget(
        buildStandardUrl(
            userRolesFullUrl,
            usersRolesGetUsersPathPart,
            usersRolesUserRoleQueryParameterName to unifiedRequester.encodeUrlQueryValue(serializer, userRole)
        ),
        UsersIdsSerializer
    )

    override suspend fun getRoles(
        userId: UserId
    ): List<T> = unifiedRequester.uniget(
        buildStandardUrl(
            userRolesFullUrl,
            usersRolesGetRolesPathPart,
            usersRolesUserIdQueryParameterName to unifiedRequester.encodeUrlQueryValue(UserId.serializer(), userId)
        ),
        userRolesSerializer
    )

    override suspend fun contains(
        userId: UserId,
        userRole: T
    ): Boolean = unifiedRequester.uniget(
        buildStandardUrl(
            userRolesFullUrl,
            usersRolesContainsPathPart,
            usersRolesUserIdQueryParameterName to unifiedRequester.encodeUrlQueryValue(UserId.serializer(), userId),
            usersRolesUserRoleQueryParameterName to unifiedRequester.encodeUrlQueryValue(serializer, userRole)
        ),
        Boolean.serializer()
    )

    override suspend fun containsAny(
        userId: UserId,
        userRoles: List<T>
    ): Boolean = unifiedRequester.uniget(
        buildStandardUrl(
            userRolesFullUrl,
            usersRolesContainsAnyPathPart,
            usersRolesUserIdQueryParameterName to unifiedRequester.encodeUrlQueryValue(UserId.serializer(), userId),
            usersRolesUserRoleQueryParameterName to unifiedRequester.encodeUrlQueryValue(userRolesSerializer, userRoles)
        ),
        Boolean.serializer()
    )
}
