package dev.inmo.postssystem.features.roles.client

import dev.inmo.postssystem.features.roles.common.*
import dev.inmo.postssystem.features.users.common.UserId
import dev.inmo.micro_utils.ktor.client.UnifiedRequester
import dev.inmo.micro_utils.ktor.common.buildStandardUrl
import kotlinx.serialization.KSerializer
import kotlinx.serialization.builtins.serializer

class WriteClientUsersRolesStorage<T : UserRole>(
    private val baseUrl: String,
    private val unifiedRequester: UnifiedRequester,
    private val serializer: KSerializer<T>
) : WriteUsersRolesStorage<T> {
    private val wrapperSerializer = UserRolesStorageIncludeExcludeWrapper.serializer(
        serializer
    )
    private val userRolesFullUrl = buildStandardUrl(
        baseUrl,
        usersRolesRootPathPart
    )
    private val includeFullUrl = buildStandardUrl(
        userRolesFullUrl,
        usersRolesIncludePathPart
    )
    private val excludeFullUrl = buildStandardUrl(
        userRolesFullUrl,
        usersRolesExcludePathPart
    )

    override suspend fun include(
        userId: UserId,
        userRole: T
    ): Boolean = unifiedRequester.unipost(
        includeFullUrl,
        wrapperSerializer to UserRolesStorageIncludeExcludeWrapper(
            userId, userRole
        ),
        Boolean.serializer()
    )

    override suspend fun exclude(
        userId: UserId,
        userRole: T
    ): Boolean  = unifiedRequester.unipost(
        excludeFullUrl,
        wrapperSerializer to UserRolesStorageIncludeExcludeWrapper(
            userId, userRole
        ),
        Boolean.serializer()
    )
}
