package dev.inmo.postssystem.features.roles.common

import kotlinx.serialization.*
import kotlinx.serialization.builtins.serializer
import kotlinx.serialization.descriptors.SerialDescriptor
import kotlinx.serialization.encoding.*
import kotlinx.serialization.json.*

@Serializable(UserRoleSerializer::class)
interface UserRole { // temporarily made as class while interfaces are bugged
    companion object {
        fun serializer() = UserRoleSerializer
    }
}

@Serializable
data class UnknownUserRole(val originalJson: JsonElement) : UserRole

@Serializer(UserRole::class)
object UserRoleSerializer : KSerializer<UserRole> {
    private val userRoleFormat = Json { ignoreUnknownKeys = true }
    private const val keyField = "key"
    private const val valueField = "value"
    private val serializers = mutableMapOf<String, KSerializer<out UserRole>>()
    override val descriptor: SerialDescriptor = String.serializer().descriptor

    @InternalSerializationApi
    override fun deserialize(decoder: Decoder): UserRole {
        return if (decoder is JsonDecoder) {
            val originalJson = decoder.decodeJsonElement().jsonObject
            val type = originalJson[keyField]?.jsonPrimitive ?.content
            return if (type == null || !serializers.containsKey(type)) {
                UnknownUserRole(originalJson)
            } else {
                userRoleFormat.decodeFromJsonElement(
                    serializers.getValue(type),
                    originalJson[valueField] ?: buildJsonObject {  }
                )
            }
        } else {
            val encoded = decoder.decodeString()
            userRoleFormat.decodeFromString(this, encoded)
        }
    }

    @InternalSerializationApi
    private fun <T : UserRole> T.toJson(): JsonElement {
        return userRoleFormat.encodeToJsonElement(this::class.serializer() as KSerializer<T>, this)
    }

    @InternalSerializationApi
    override fun serialize(encoder: Encoder, value: UserRole) {
        if (encoder is JsonEncoder) {
            if (value is UnknownUserRole) {
                encoder.encodeJsonElement(value.originalJson)
            } else {
                val valueSerializer = value::class.serializer()
                val type = serializers.keys.first { serializers[it] == valueSerializer }
                encoder.encodeJsonElement(
                    buildJsonObject {
                        put(keyField, type)
                        put(valueField, value.toJson())
                    }
                )
            }
        } else {
            encoder.encodeString(
                userRoleFormat.encodeToString(this, value)
            )
        }
    }

    fun <T : UserRole> includeSerializer(
        type: String,
        kSerializer: KSerializer<T>
    ) { serializers[type] = kSerializer }

    fun excludeSerializer(type: String) {
        serializers.remove(type)
    }
}
