package dev.inmo.postssystem.features.roles.common.keyvalue

import dev.inmo.postssystem.features.common.common.default
import dev.inmo.postssystem.features.roles.common.ReadUsersRolesStorage
import dev.inmo.postssystem.features.roles.common.UserRole
import dev.inmo.postssystem.features.users.common.UserId
import dev.inmo.micro_utils.pagination.changeResults
import dev.inmo.micro_utils.pagination.utils.getAllByWithNextPaging
import dev.inmo.micro_utils.repos.ReadKeyValuesRepo
import kotlinx.serialization.KSerializer
import kotlinx.serialization.StringFormat
import kotlinx.serialization.json.Json

open class ReadKeyValueUsersRolesStorage<T : UserRole>(
    private val keyValuesRepo: ReadKeyValuesRepo<Long, String>,
    private val serializer: KSerializer<T>,
    private val format: StringFormat = defaultJson
) : ReadUsersRolesStorage<T> {
    override suspend fun getUsers(userRole: T): List<UserId> {
        val serialized = format.encodeToString(serializer, userRole)

        return keyValuesRepo.getAllByWithNextPaging {
            keys(serialized, it).let { paginationResult ->
                paginationResult.changeResults(
                    paginationResult.results.map { UserId(it) }
                )
            }
        }
    }

    override suspend fun getRoles(userId: UserId): List<T> {
        return keyValuesRepo.getAllByWithNextPaging {
            get(userId.long, it).let { paginationResult ->
                paginationResult.changeResults(
                    paginationResult.results.map { serialized ->
                        format.decodeFromString(serializer, serialized)
                    }
                )
            }
        }
    }

    override suspend fun contains(userId: UserId, userRole: T): Boolean {
        val serialized = format.encodeToString(serializer, userRole)

        return keyValuesRepo.contains(userId.long, serialized)
    }

    override suspend fun containsAny(userId: UserId, userRoles: List<T>): Boolean {
        return userRoles.any {
            contains(userId, it)
        }
    }

    companion object {
        internal val defaultJson = Json.default
    }
}

