package dev.inmo.postssystem.features.roles.common.keyvalue

import dev.inmo.postssystem.features.roles.common.UserRole
import dev.inmo.postssystem.features.roles.common.WriteUsersRolesStorage
import dev.inmo.postssystem.features.users.common.UserId
import dev.inmo.micro_utils.repos.*
import kotlinx.serialization.KSerializer
import kotlinx.serialization.StringFormat

open class WriteKeyValueUsersRolesStorage<T : UserRole>(
    private val keyValuesRepo: WriteKeyValuesRepo<Long, String>,
    private val serializer: KSerializer<T>,
    private val format: StringFormat = ReadKeyValueUsersRolesStorage.defaultJson
) : WriteUsersRolesStorage<T> {
    override suspend fun include(userId: UserId, userRole: T): Boolean {
        return runCatching {
            keyValuesRepo.add(
                userId.long,
                format.encodeToString(serializer, userRole)
            )
            true
        }.getOrElse { false }
    }

    override suspend fun exclude(userId: UserId, userRole: T): Boolean {
        return runCatching {
            keyValuesRepo.remove(
                userId.long,
                format.encodeToString(serializer, userRole)
            )
            true
        }.getOrElse { false }
    }
}
