package dev.inmo.postssystem.features.roles.server

import dev.inmo.postssystem.features.roles.common.*
import dev.inmo.postssystem.features.users.common.User
import io.ktor.application.ApplicationCall

interface RolesChecker<T : UserRole> {
    val key: String

    suspend operator fun ApplicationCall.invoke(
        usersRolesStorage: ReadUsersRolesStorage<T>,
        user: User
    ): Boolean

    companion object {
        fun <T : UserRole> default(
            key: String,
            role: T
        ): RolesChecker<T> = object : RolesChecker<T> {
            override val key: String
                get() = key

            override suspend fun ApplicationCall.invoke(
                usersRolesStorage: ReadUsersRolesStorage<T>,
                user: User
            ): Boolean = usersRolesStorage.contains(user.id, role)
        }
    }
}
