package dev.inmo.postssystem.features.roles.server

import dev.inmo.postssystem.features.roles.common.UserRole
import dev.inmo.postssystem.features.roles.common.UsersRolesStorage
import dev.inmo.postssystem.features.users.common.UserId

class UsersRolesAggregator(
    private val otherStorages: List<UsersRolesStorageHolder<*>>
) : UsersRolesStorage<UserRole> {
    private val otherStoragesByClass = otherStorages.associateBy { it.kclass }

    override suspend fun getUsers(userRole: UserRole): List<UserId> {
        return otherStoragesByClass[userRole::class] ?.getUsers(userRole) ?: emptyList()
    }

    override suspend fun getRoles(userId: UserId): List<UserRole> = otherStorages.flatMap { it.getRoles(userId) }

    override suspend fun contains(userId: UserId, userRole: UserRole): Boolean {
        return otherStoragesByClass[userRole::class] ?.contains(userId, userRole) ?: false
    }

    override suspend fun containsAny(userId: UserId, userRoles: List<UserRole>): Boolean {
        return userRoles.any {
            contains(userId, it)
        }
    }

    override suspend fun include(
        userId: UserId,
        userRole: UserRole
    ): Boolean = otherStoragesByClass[userRole::class] ?.include(userId, userRole) ?: false

    override suspend fun exclude(
        userId: UserId,
        userRole: UserRole
    ): Boolean {
        return otherStoragesByClass[userRole::class] ?.exclude(userId, userRole) ?: false
    }
}
