package dev.inmo.postssystem.features.roles.server

import dev.inmo.postssystem.features.roles.common.UserRole
import dev.inmo.postssystem.features.roles.common.UsersRolesStorage
import dev.inmo.postssystem.features.users.common.UserId
import dev.inmo.micro_utils.common.*
import kotlin.reflect.KClass

data class UsersRolesStorageHolder<T : UserRole>(
    val kclass: KClass<T>,
    val storage: UsersRolesStorage<T>
) {
    private suspend fun <R> doIfRelevant(
        userRole: UserRole,
        block: suspend (T) -> R
    ): Optional<R> = if (kclass.isInstance(userRole)) {
        block(userRole as T).optional
    } else {
        Optional.absent()
    }

    suspend fun getUsers(userRole: UserRole): List<UserId>? = doIfRelevant(userRole) {
        storage.getUsers(it)
    }.dataOrNull()

    suspend fun getRoles(userId: UserId): List<UserRole> = storage.getRoles(userId)

    suspend fun contains(userId: UserId, userRole: UserRole): Boolean? = doIfRelevant(userRole) {
        storage.contains(userId, it)
    }.dataOrNull()

    suspend fun include(
        userId: UserId,
        userRole: UserRole
    ): Boolean? = doIfRelevant(userRole) {
        storage.include(userId, it)
    }.dataOrNull()

    suspend fun exclude(userId: UserId, userRole: UserRole): Boolean? = doIfRelevant(userRole) {
        storage.exclude(userId, it)
    }.dataOrNull()
}
