package dev.inmo.postssystem.features.roles.server

import dev.inmo.postssystem.features.roles.common.*
import dev.inmo.postssystem.features.users.common.UserId
import dev.inmo.micro_utils.ktor.server.*
import dev.inmo.micro_utils.ktor.server.configurators.ApplicationRoutingConfigurator
import io.ktor.application.call
import io.ktor.auth.authenticate
import io.ktor.routing.*
import kotlinx.serialization.KSerializer
import kotlinx.serialization.builtins.ListSerializer
import kotlinx.serialization.builtins.serializer

class UsersRolesStorageReadServerRoutesConfigurator<T : UserRole>(
    private val storage: ReadUsersRolesStorage<T>,
    private val serializer: KSerializer<T>
) : ApplicationRoutingConfigurator.Element {
    private val userRolesSerializer = ListSerializer(serializer)
    override fun Route.invoke() {
        authenticate {
            route(usersRolesRootPathPart) {
                get(usersRolesGetUsersPathPart) {
                    val userRole = call.decodeUrlQueryValueOrSendError(usersRolesUserRoleQueryParameterName, serializer)
                        ?: return@get
                    call.unianswer(
                        UsersIdsSerializer,
                        storage.getUsers(userRole)
                    )
                }

                get(usersRolesGetRolesPathPart) {
                    val userId =
                        call.decodeUrlQueryValueOrSendError(usersRolesUserIdQueryParameterName, UserId.serializer())
                            ?: return@get
                    call.unianswer(
                        userRolesSerializer,
                        storage.getRoles(userId)
                    )
                }

                get(usersRolesContainsPathPart) {
                    val userId = call.decodeUrlQueryValueOrSendError(
                        usersRolesUserIdQueryParameterName,
                        UserId.serializer()
                    ) ?: return@get
                    val userRole = call.decodeUrlQueryValueOrSendError(
                        usersRolesUserRoleQueryParameterName,
                        serializer
                    ) ?: return@get
                    call.unianswer(
                        Boolean.serializer(),
                        storage.contains(userId, userRole)
                    )
                }

                get(usersRolesContainsAnyPathPart) {
                    val userId = call.decodeUrlQueryValueOrSendError(
                        usersRolesUserIdQueryParameterName,
                        UserId.serializer()
                    ) ?: return@get
                    val userRoles = call.decodeUrlQueryValueOrSendError(
                        usersRolesUserRoleQueryParameterName,
                        userRolesSerializer
                    ) ?: return@get
                    call.unianswer(
                        Boolean.serializer(),
                        storage.containsAny(userId, userRoles)
                    )
                }
            }
        }
    }
}
