package dev.inmo.postssystem.features.roles.server

import dev.inmo.postssystem.features.roles.common.*
import dev.inmo.micro_utils.ktor.server.*
import dev.inmo.micro_utils.ktor.server.configurators.ApplicationRoutingConfigurator
import io.ktor.application.call
import io.ktor.auth.authenticate
import io.ktor.routing.*
import kotlinx.serialization.KSerializer
import kotlinx.serialization.builtins.serializer

class UsersRolesStorageWriteServerRoutesConfigurator<T : UserRole>(
    private val storage: WriteUsersRolesStorage<T>,
    private val serializer: KSerializer<T>,
    private val includeAuthKey: String,
    private val excludeAuthKey: String = includeAuthKey
) : ApplicationRoutingConfigurator.Element {
    override fun Route.invoke() {
        route(usersRolesRootPathPart) {
            val wrapperSerializer = UserRolesStorageIncludeExcludeWrapper.serializer(
                serializer
            )
            authenticate(includeAuthKey) {
                post(usersRolesIncludePathPart) {
                    val wrapper = call.uniload(wrapperSerializer)

                    call.unianswer(
                        Boolean.serializer(),
                        storage.include(
                            wrapper.userId,
                            wrapper.userRole
                        )
                    )
                }
            }
            authenticate(excludeAuthKey) {
                post(usersRolesExcludePathPart) {
                    val wrapper = call.uniload(wrapperSerializer)

                    call.unianswer(
                        Boolean.serializer(),
                        storage.exclude(
                            wrapper.userId,
                            wrapper.userRole
                        )
                    )
                }
            }
        }
    }
}
